"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const oracledb_1 = require("oracledb");
const moment = require("moment");
const Parameters_1 = require("./Parameters");
const PARAMETERS_SEPARATOR = '/*PARAMETERS END*/';
const BEGIN = 'BEGIN';
class BindsDebugger {
    static getDebugSql(query, bindParams) {
        if (Parameters_1.default.config === undefined) {
            throw new Error('Config must be defined for sql debugger.');
        }
        let debugSql = query;
        try {
            for (const parameter of Object.keys(bindParams || {})) {
                debugSql = this.replaceParameter(debugSql, parameter, bindParams[parameter]);
            }
        }
        catch (error) {
            debugSql = `Error preparing sql debug query: ${error.message}`;
        }
        return debugSql;
    }
    static replaceParameter(query, parameter, bindParam) {
        let replacedQuery;
        if (bindParam instanceof Object && !(bindParam instanceof Date)) {
            replacedQuery = this.replaceComplexParameter(query, parameter, bindParam);
        }
        else {
            replacedQuery = this.replaceSimpleParameter(query, parameter, bindParam);
        }
        return replacedQuery;
    }
    static replaceComplexParameter(query, parameter, bindParam) {
        if (bindParam instanceof Date) {
            return this.replaceSimpleParameter(query, parameter, bindParam);
        }
        if (bindParam instanceof Array) {
            return this.replaceArrayParameter(query, parameter, bindParam);
        }
        if (bindParam.constructor.name.indexOf('DbObject') !== -1) {
            return this.replaceDBObjectParameter(query, parameter, bindParam);
        }
        if (bindParam.dir === oracledb_1.BIND_OUT && bindParam.type === oracledb_1.CURSOR) {
            return this.replaceDBObjectCursorParameter(query, parameter, bindParam);
        }
        throw new Error('Uknown type provided for binds as complex parameter');
    }
    static replaceDBObjectCursorParameter(query, parameter, bindParam) {
        if (!this.convertCursor) {
            return query;
        }
        const { declare, begin, end } = this.splitQuery(query);
        return `${declare}
        ${parameter} sys_refcursor;
        ${begin}
        ${PARAMETERS_SEPARATOR}
        ${end.replace(`:${parameter}`, parameter)}`;
    }
    static replaceDBObjectParameterElement(query, parameter, bindParam) {
        const { declare, begin, end } = this.splitQuery(query);
        return `${declare}
        ${parameter} := ${bindParam.__proto__.fqn}(${this.getObjectParametersString(bindParam).join(',')});
        ${begin}
        ${PARAMETERS_SEPARATOR}
        ${end.replace(`:${parameter}`, parameter)}`;
    }
    static replaceDBObjectParameter(query, parameter, bindParam) {
        if (bindParam.isCollection) {
            const { variablesDeclare, extend } = this.prepareTypeVariables(parameter, bindParam);
            let values;
            if (['VARCHAR2', 'NUMBER', 'FLOAT', 'DATE'].includes(bindParam.elementTypeName)) {
                values = this.prepareDBObjectCollectionElementSimpleValue(parameter, bindParam);
            }
            else {
                values = this.prepareDBObjectCollectionElementValue(parameter, bindParam);
            }
            return this.formDBObjectParameterReplacedQuery({ query, parameter, variablesDeclare, extend, values });
        }
        else {
            return this.replaceDBObjectParameterElement(query, parameter, bindParam);
        }
    }
    static formDBObjectParameterReplacedQuery(inputs) {
        const { declare, begin, end } = this.splitQuery(inputs.query);
        return `${declare}
        ${inputs.variablesDeclare}
        ${begin}
        ${inputs.extend}
        ${inputs.values.join('\n\t')}
        ${PARAMETERS_SEPARATOR}
        ${end.replace(`:${inputs.parameter}`, inputs.parameter)}`;
    }
    static prepareDBObjectCollectionElementSimpleValue(parameter, bindParam) {
        const values = [];
        for (let i = 0; i < bindParam.length; i++) {
            const element = bindParam.getElement(i);
            values.push(`${parameter}(${i + 1}) := ${this.prepareBindParamValueForReplace(element)};`);
        }
        return values;
    }
    static prepareDBObjectCollectionElementValue(parameter, bindParam) {
        const values = [];
        const elementTypeName = bindParam.elementTypeName;
        for (let i = 0; i < bindParam.length; i++) {
            const element = bindParam.getElement(i);
            const parameters = this.getObjectParametersString(element);
            values.push(`${parameter}(${i + 1}) := ${elementTypeName}(${parameters.join(',')});`);
        }
        return values;
    }
    static getObjectParametersString(element) {
        const parameters = [];
        for (const parameterName in element.attributes) {
            parameters.push(`${parameterName} => ${this.prepareBindParamValueForReplace(element[parameterName])}`);
        }
        return parameters;
    }
    static splitQuery(query) {
        return {
            declare: this.getSplitQueryDeclare(query),
            begin: this.getSplitQueryBegin(query),
            end: this.getSplitQueryEnd(query)
        };
    }
    static getSplitQueryDeclare(query) {
        const queryUpperCase = query.toUpperCase();
        const beginStart = queryUpperCase.indexOf(BEGIN);
        if (beginStart === -1) {
            throw new Error(`No begin statement in query`);
        }
        else if (beginStart === 0) {
            return 'DECLARE';
        }
        else {
            return query.slice(0, beginStart);
        }
    }
    static getSplitQueryBegin(query) {
        const queryUpperCase = query.toUpperCase();
        const endStart = queryUpperCase.indexOf(PARAMETERS_SEPARATOR);
        if (endStart === -1) {
            return `BEGIN\n`;
        }
        else {
            return query.slice(queryUpperCase.indexOf(BEGIN), endStart);
        }
    }
    static getSplitQueryEnd(query) {
        const queryUpperCase = query.toUpperCase();
        const endStart = queryUpperCase.indexOf(PARAMETERS_SEPARATOR);
        if (endStart === -1) {
            return query.slice(queryUpperCase.indexOf(BEGIN) + BEGIN.length);
        }
        else {
            return query.slice(endStart + PARAMETERS_SEPARATOR.length);
        }
    }
    static prepareTypeVariables(parameter, bindParam) {
        return {
            variablesDeclare: `${parameter} ${bindParam.fqn} := ${bindParam.fqn}();`,
            extend: `${parameter}.extend(${bindParam.length});`
        };
    }
    static replaceArrayParameter(query, parameter, bindParam) {
        throw new Error('Method "replaceArrayParameter" not implemented.');
    }
    static replaceSimpleParameter(query, parameter, bindParam) {
        const value = this.prepareBindParamValueForReplace(bindParam);
        return query.replace(`:${parameter}`, value);
    }
    static prepareBindParamValueForReplace(bindParamValue) {
        switch (typeof bindParamValue) {
            case 'number':
                return `${bindParamValue}`;
            case 'string':
                return `'${bindParamValue.replace(new RegExp(`'`, 'g'), `''`)}'`;
            default:
                if (bindParamValue === null) {
                    // tslint:disable-next-line: no-null-keyword
                    return null;
                }
                if (bindParamValue instanceof Date) {
                    return `to_date('${moment(bindParamValue).format(Parameters_1.default.config.db.momentDateFormat)}','${process.env.NLS_DATE_FORMAT}')`;
                }
                return bindParamValue.toString();
        }
    }
}
exports.default = BindsDebugger;
BindsDebugger.convertCursor = false;
//# sourceMappingURL=BindsDebugger.js.map