"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OracleDBHelper = void 0;
const moment = require("moment");
const oracledb = require("oracledb");
const Errors_1 = require("./supportObjects/Errors");
const CallParameters_1 = require("./CallParameters");
const CallParametersDataParameter_1 = require("./CallParametersDataParameter");
const Parameters_1 = require("./Parameters");
var SortDirection;
(function (SortDirection) {
    SortDirection["ASC"] = "ASC";
    SortDirection["DESC"] = "DESC";
})(SortDirection || (SortDirection = {}));
const FilterOperatorAliases = {
    'gt': '>',
    'eq': '=',
    'lt': '<',
    'lte': '<=',
    'gte': '>=',
    'notin': 'not in',
    'isnot': 'is not'
};
var FilterOperators;
(function (FilterOperators) {
    FilterOperators["GREATER_THAN"] = ">";
    FilterOperators["EQUAL"] = "=";
    FilterOperators["NOT_EQUAL"] = "!=";
    FilterOperators["LESS_THAN"] = "<";
    FilterOperators["LESS_EQUAL_THAN"] = "<=";
    FilterOperators["GREATER_EQUAL_THAN"] = ">=";
    FilterOperators["NOT_IN"] = "not in";
    FilterOperators["LIKE"] = "like";
    FilterOperators["IS"] = "is";
    FilterOperators["IN"] = "in";
    FilterOperators["OR"] = "or";
    FilterOperators["IS_NOT"] = "is not";
})(FilterOperators || (FilterOperators = {}));
/**
 * Helper for perform operations on Oracle database
 *
 */
class OracleDBHelper {
    static prepareCursorOutputBindParameter(variableName = 'curs') {
        const bindParameters = {};
        bindParameters[variableName] = { dir: oracledb.BIND_OUT, type: oracledb.CURSOR };
        return bindParameters;
    }
    static async createConnectionPool() {
        Parameters_1.default.logger.debug('Creating connection pool');
        try {
            const pool = await oracledb.createPool({
                user: Parameters_1.default.config.db.user,
                password: Parameters_1.default.config.db.password,
                connectString: Parameters_1.default.config.db.connectString,
                poolMin: Parameters_1.default.config.db.minConnectionsInPool,
                poolMax: Parameters_1.default.config.db.maxConnectionsInPool
            });
            Parameters_1.default.logger.debug('Connection Pool created.');
            return pool;
        }
        catch (error) {
            Parameters_1.default.logger.error(`Connection Pool creation failed. ${error.message}`);
            this.closeConnectionPool();
            throw error;
        }
    }
    static closeConnectionPool() {
        try {
            oracledb.getPool().close(60);
        }
        catch (err) {
            Parameters_1.default.logger.error(err.message);
            process.exit(1);
        }
    }
    prepareGetCallParameters(resourceConfig, requestParameters) {
        const callParameters = new CallParameters_1.default();
        callParameters.procedureFunctionName = resourceConfig.getProcedure;
        callParameters.data = this.prepareGetDataParameters(requestParameters, resourceConfig.idFields[0], resourceConfig.dateFields);
        return callParameters;
    }
    prepareGetDataParameters(requestParameters, idField, dateFields) {
        const parameters = [];
        parameters.push(new CallParametersDataParameter_1.default('p_offset', requestParameters.offset));
        parameters.push(new CallParametersDataParameter_1.default('p_limit', requestParameters.limit));
        parameters.push(new CallParametersDataParameter_1.default('p_sort', this.prepareSortParameterValue(requestParameters.sort)));
        parameters.push(this.prepareFieldsParameter(requestParameters.fields));
        parameters.push(this.prepareFilterParameter(requestParameters, idField, dateFields));
        return parameters;
    }
    prepareFieldsParameter(fields) {
        const parameter = new CallParametersDataParameter_1.default('p_fields');
        parameter.isComplexType = true;
        parameter.type = 'varchar2_t';
        parameter.value = [];
        for (const field of fields) {
            parameter.value.push(field);
        }
        return parameter;
    }
    prepareFilterParameter(requestParameters, idField, dateFields) {
        const parameter = new CallParametersDataParameter_1.default('P_filter');
        parameter.isComplexType = true;
        parameter.type = 'clauses_t';
        let filter = [];
        if (requestParameters.itemId !== undefined) {
            filter.push({
                COL_NAME: idField,
                CLAUSE: '=',
                COL_VALUE: requestParameters.itemId.toString()
            });
        }
        else if (typeof requestParameters.filter !== 'undefined') {
            filter = this.prepareFilterParameterValues(requestParameters.filter, dateFields);
        }
        parameter.value = filter;
        return parameter;
    }
    removeEmptyListFilters(filter) {
        const filteredFilter = [];
        for (const f of filter) {
            if (f.operator == 'in' && (!f.value || f.value.length === 0)) {
                continue;
            }
            filteredFilter.push(f);
        }
        return filteredFilter;
    }
    prepareFilterParameterValues(filters, dateFields) {
        filters = this.removeEmptyListFilters(filters);
        const { betweenFiltersPrepared, remainingFilterItems } = this.getBetweenFilters(filters);
        const filterParameterValues = betweenFiltersPrepared;
        for (const filterItem of remainingFilterItems) {
            filterParameterValues.push(this.prepareFilterParameterValue(filterItem, (dateFields.indexOf(filterItem.property.toUpperCase()) >= 0)));
        }
        return filterParameterValues;
    }
    prepareFilterParameterValue(filterItem, isDateField) {
        try {
            const filterItemValue = this.prepareFilterItemValue(filterItem);
            const operator = this.prepareFilterParameterValueOperator(filterItem.operator.toLowerCase());
            const property = filterItem.property.toUpperCase();
            let filterColumnName = property;
            let filterColumnValue = filterItemValue;
            let filterClause = operator;
            if (isDateField) {
                if (operator === FilterOperators.IS || operator === FilterOperators.LIKE ||
                    operator === FilterOperators.EQUAL) {
                    filterColumnName = `TRUNC(${property})`;
                    filterClause = FilterOperators.EQUAL;
                    filterColumnValue = `${filterItemValue}`;
                }
                else if (operator === FilterOperators.GREATER_THAN || operator === FilterOperators.GREATER_EQUAL_THAN ||
                    operator === FilterOperators.LESS_THAN || operator === FilterOperators.LESS_EQUAL_THAN) {
                    filterColumnName = '';
                    filterClause = '';
                    filterColumnValue = `trunc(${property}) ${operator} to_date(${this.getQuotedString(filterItemValue)}, ${this.getQuotedString(Parameters_1.default.config.db.dateFormat)})`;
                }
            }
            else {
                if (operator === FilterOperators.IN || operator === FilterOperators.NOT_IN) {
                    filterColumnName = '';
                    filterClause = '';
                    filterColumnValue = `and ${property} ${operator} ${filterItemValue}`;
                }
                else if (operator === FilterOperators.LIKE) {
                    filterColumnName = `upper(${filterColumnName})`;
                    filterColumnValue = `%${filterItemValue.toUpperCase()}%`;
                }
                else if ((filterItemValue === null) && (operator === FilterOperators.IS || operator === FilterOperators.IS_NOT)) {
                    filterClause = '';
                    filterColumnValue = `upper(${filterColumnName}) ${operator} null`;
                    filterColumnName = '';
                }
            }
            return {
                COL_VALUE: filterColumnValue != undefined ? filterColumnValue.toString() : filterColumnValue,
                COL_NAME: filterColumnName,
                CLAUSE: filterClause
            };
        }
        catch (error) {
            throw new Errors_1.BadInputError('Filter is invalid. ' + error.message);
        }
    }
    prepareFilterParameterValueOperator(operator) {
        if (Object.keys(FilterOperatorAliases).indexOf(operator) === -1) {
            return operator;
        }
        return FilterOperatorAliases[operator];
    }
    prepareFilterItemValue(filterItem) {
        let filterItemValue;
        if (filterItem.value instanceof Array) {
            filterItemValue = '(';
            let first = true;
            for (const filterItemListValue of filterItem.value) {
                if (first) {
                    first = false;
                }
                else {
                    filterItemValue += ',';
                }
                filterItemValue += this.getQuotedString(filterItemListValue);
            }
            filterItemValue += ')';
        }
        else {
            filterItemValue = filterItem.value;
        }
        return filterItemValue;
    }
    preparePotencialBetweenItems(filterItems) {
        const betweenPotencialItems = {};
        const GREATER_THAN = 'gt';
        const LESS_THAN = 'lt';
        for (const i in filterItems) {
            const filter = filterItems[i];
            if ([GREATER_THAN, LESS_THAN].indexOf(filter.operator) === -1) {
                continue;
            }
            if (betweenPotencialItems[filter.property] === undefined) {
                betweenPotencialItems[filter.property] = { greaterThan: undefined, lessThan: undefined, filterIndex: [] };
            }
            betweenPotencialItems[filter.property].filterIndex.push(i);
            if (filter.operator === GREATER_THAN) {
                betweenPotencialItems[filter.property].greaterThan = filter.value;
            }
            else if (filter.operator === LESS_THAN) {
                betweenPotencialItems[filter.property].lessThan = filter.value;
            }
        }
        return betweenPotencialItems;
    }
    removeBetweenFiltersFromList(filterItems, property) {
        return filterItems.filter((value) => {
            return value.property !== property;
        });
    }
    getBetweenFilters(filterItems) {
        const betweenPotencialItems = this.preparePotencialBetweenItems(filterItems);
        const betweenFiltersPrepared = [];
        let remainingFilterItems = filterItems;
        for (const property in betweenPotencialItems) {
            const betweenPotencialItem = betweenPotencialItems[property];
            if (betweenPotencialItem.lessThan === undefined || betweenPotencialItem.greaterThan === undefined) {
                continue;
            }
            this.prepareBetweenFilterDates(betweenPotencialItem);
            betweenFiltersPrepared.push({
                COL_NAME: '',
                CLAUSE: '',
                COL_VALUE: `${property} BETWEEN ${betweenPotencialItem.greaterThan} and ${betweenPotencialItem.lessThan}`
            });
            remainingFilterItems = this.removeBetweenFiltersFromList(remainingFilterItems, property);
        }
        return { betweenFiltersPrepared, remainingFilterItems };
    }
    prepareBetweenFilterDates(betweenItem) {
        if (moment(betweenItem.lessThan, Parameters_1.default.config.db.momentDateWithoutTimeFormat).isValid()) {
            betweenItem.lessThan = this.getQuotedString(betweenItem.lessThan);
        }
        if (moment(betweenItem.greaterThan, Parameters_1.default.config.db.momentDateWithoutTimeFormat).isValid()) {
            betweenItem.greaterThan = this.getQuotedString(betweenItem.greaterThan);
        }
    }
    prepareSortParameterValue(sortValues) {
        const sort = [];
        for (const sortValue of sortValues) {
            const direction = sortValue.direction.toUpperCase();
            if (!(direction in SortDirection)) {
                throw new Errors_1.BadInputError(`Invalid sort direction "${sortValue.direction}" provided for "${sortValue.property}"`);
            }
            sort.push(`${sortValue.property} ${direction}`);
        }
        return sort.join(',');
    }
    checkSaveDataParams(reqConfig, params, isUpdate) {
        if (params === undefined) {
            throw new Errors_1.BadInputError(`No request body provided for ${reqConfig.itemName} item`);
        }
        const idValue = params[reqConfig.idFields[0]];
        if (idValue === undefined && isUpdate) {
            throw new Errors_1.BadInputError(`Id value should be provided in the request for updating ${reqConfig.itemName} item details`);
        }
        else if (idValue > 0 && !isUpdate) {
            throw new Errors_1.BadInputError(`Id value shouldn't be provided in the request for storing ${reqConfig.itemName} item details`);
        }
        else if (reqConfig.versionField !== undefined && idValue > 0 && params[reqConfig.versionField] === undefined) {
            throw new Errors_1.BadInputError(`Version number should be provided in the request for updating ${reqConfig.itemName} item details`);
        }
    }
    prepareSaveDataParameters(dataFields) {
        if (Object.keys(dataFields).length === 0) {
            throw new Errors_1.BadInputError('No data provided for save');
        }
        const parameters = [];
        for (const key of Object.keys(dataFields)) {
            const parameterName = `p_${key.toLowerCase()}`;
            parameters.push(new CallParametersDataParameter_1.default(parameterName, dataFields[key]));
        }
        return parameters;
    }
    prepareSaveCallParameters(functionName, dataFields) {
        const callParameters = new CallParameters_1.default();
        callParameters.procedureFunctionName = functionName;
        callParameters.data = this.prepareSaveDataParameters(dataFields);
        return callParameters;
    }
    prepareDeleteCallParameters(functionName, idField, id) {
        const callParameters = new CallParameters_1.default();
        callParameters.procedureFunctionName = functionName;
        callParameters.data = [new CallParametersDataParameter_1.default(`p_${idField}`, id)];
        return callParameters;
    }
    getQuotedString(string) {
        return `'${string}'`;
    }
}
exports.OracleDBHelper = OracleDBHelper;
// tslint:disable-next-line: no-null-keyword
OracleDBHelper.NULL = null;
//# sourceMappingURL=OracleDBHelper.js.map