"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OracleDB = void 0;
const ProcedureFunctionHelper_1 = require("./ProcedureFunctionHelper");
const oracledb = require("oracledb");
const BindsDebugger_1 = require("./BindsDebugger");
const OracleDBHelper_1 = require("./OracleDBHelper");
const defaultLogger_1 = require("./supportObjects/defaultLogger");
const Parameters_1 = require("./Parameters");
oracledb.outFormat = oracledb.OUT_FORMAT_OBJECT;
oracledb.extendedMetaData = true;
oracledb.fetchAsString = [oracledb.CLOB, oracledb.DATE];
/**
 * Connects and performs operations on Oracle database
 *
 */
class OracleDB {
    constructor() {
        this._debugLog = new Array();
        this._autoCommit = true;
        if (Parameters_1.default.config === undefined) {
            throw new Error('Config must be defined for sql debugger.');
        }
        oracledb.autoCommit = this._autoCommit;
    }
    async initializeConnection() {
        Parameters_1.default.logger.debug('Initializing database connection');
        this._connection = await oracledb.getConnection();
        this.setTimeout();
    }
    setTimeout(timeoutInSeconds) {
        if (timeoutInSeconds === undefined) {
            this._connection.callTimeout = ((Parameters_1.default.config.db.timeoutInSeconds) ? Parameters_1.default.config.db.timeoutInSeconds : 30) * 1000;
        }
        else {
            this._connection.callTimeout = timeoutInSeconds * 1000;
        }
    }
    prepareFieldTypesFromMetaData(metaData) {
        const fieldTypes = {};
        for (const metaDataItem of metaData) {
            fieldTypes[metaDataItem.name] = metaDataItem.dbTypeName;
        }
        return fieldTypes;
    }
    fixNumerValues(row, metaData) {
        defaultLogger_1.default.debug(row);
        for (const key of Object.keys(row)) {
            if (metaData[key] === 'NUMBER' && row[key] !== OracleDBHelper_1.OracleDBHelper.NULL) {
                row[key] = parseFloat(row[key]);
            }
        }
        return row;
    }
    async getDataFromCursorInResult(resultSet, cursorName = 'curs', rowsCountToRead) {
        if (!resultSet.outBinds) {
            throw new Error(`Result does not contain outBinds`);
        }
        if (!resultSet.outBinds[cursorName]) {
            throw new Error(`Result outBinds Cursor ${cursorName} does not exist`);
        }
        const rows = [];
        const metaData = resultSet.outBinds[cursorName].metaData;
        let row;
        while (row = await resultSet.outBinds[cursorName].getRow()) {
            rows.push(this.fixNumerValues(row, this.prepareFieldTypesFromMetaData(metaData)));
        }
        resultSet.outBinds[cursorName].close();
        return rows;
    }
    logExecuteDetails(bindParameters, query) {
        const logBinds = `Binds: '${JSON.stringify(bindParameters)}'`;
        const logSql = `Raw sql: '${query}'`;
        const logDebugSql = `Debug sql: '${BindsDebugger_1.default.getDebugSql(query, bindParameters)}'`;
        const separator = `========================`;
        this._debugLog.push(logSql);
        this._debugLog.push(logBinds);
        this._debugLog.push(logDebugSql.replace(/\n/g, ''));
        this._debugLog.push(separator);
        Parameters_1.default.logger.debug(logBinds);
        Parameters_1.default.logger.debug(logSql);
        Parameters_1.default.logger.debug(logDebugSql);
    }
    async closeConection() {
        try {
            this._connection.close();
            this._debugLog = new Array();
            Parameters_1.default.logger.info('Database connection successfully closed');
        }
        catch (err) {
            Parameters_1.default.logger.error('Error closing database connection');
            Parameters_1.default.logger.error(err);
        }
    }
    get debugLog() {
        return this._debugLog;
    }
    execute(query, bindParams = {}, options = {}) {
        this.logExecuteDetails(bindParams, query);
        return this._connection.execute(query, bindParams, options);
    }
    set autoCommit(value) {
        this._autoCommit = value;
        oracledb.autoCommit = this._autoCommit;
    }
    async commit() {
        await this._connection.commit();
    }
    async rollback() {
        await this._connection.rollback();
    }
    async callProcedure(data) {
        const procedureFunctionHelper = new ProcedureFunctionHelper_1.default(this, data);
        return procedureFunctionHelper.execute();
    }
    getDbObjectClass(className) {
        return this._connection.getDbObjectClass(className.toUpperCase());
    }
}
exports.OracleDB = OracleDB;
//# sourceMappingURL=index.js.map